@@ -17,7 +17,6 @@
 package org.apache.logging.log4j.message;
 
 import org.apache.logging.log4j.util.PerformanceSensitive;
-import org.apache.logging.log4j.util.StringBuilderFormattable;
 import org.apache.logging.log4j.util.StringBuilders;
 
 /**
@@ -25,11 +24,10 @@ import org.apache.logging.log4j.util.StringBuilders;
  * @since 2.6
  */
 @PerformanceSensitive("allocation")
-public class ReusableObjectMessage implements ReusableMessage {
+public class ReusableObjectMessage implements ReusableMessage, ParameterVisitable, Clearable {
     private static final long serialVersionUID = 6922476812535519960L;
 
     private transient Object obj;
-    private transient String objectString;
 
     public void set(final Object object) {
         this.obj = object;
@@ -57,7 +55,7 @@ public class ReusableObjectMessage implements ReusableMessage {
      */
     @Override
     public String getFormat() {
-        return getFormattedMessage();
+        return obj instanceof String ? (String) obj : null;
     }
 
     /**
@@ -96,26 +94,45 @@ public class ReusableObjectMessage implements ReusableMessage {
     }
 
     /**
-     * This message does not have any parameters, so this method returns the specified array.
+     * This message has exactly one parameter (the object), so returns it as the first parameter in the array.
      * @param emptyReplacement the parameter array to return
      * @return the specified array
      */
     @Override
     public Object[] swapParameters(final Object[] emptyReplacement) {
+        // it's unlikely that emptyReplacement is of length 0, but if it is,
+        // go ahead and allocate the memory now;
+        // this saves an allocation in the future when this buffer is re-used
+        if (emptyReplacement.length == 0) {
+            final Object[] params = new Object[10]; // Default reusable parameter buffer size
+            params[0] = obj;
+            return params;
+        }
+        emptyReplacement[0] = obj;
         return emptyReplacement;
     }
 
     /**
-     * This message does not have any parameters so this method always returns zero.
-     * @return 0 (zero)
+     * This message has exactly one parameter (the object), so always returns one.
+     * @return 1
      */
     @Override
     public short getParameterCount() {
-        return 0;
+        return 1;
+    }
+
+    @Override
+    public <S> void forEachParameter(final ParameterConsumer<S> action, final S state) {
+        action.accept(obj, 0, state);
     }
 
     @Override
     public Message memento() {
         return new ObjectMessage(obj);
     }
+
+    @Override
+    public void clear() {
+        obj = null;
+    }
 }
