@@ -34,6 +34,7 @@ import org.apache.logging.log4j.core.config.plugins.PluginBuilderFactory;
 import org.apache.logging.log4j.core.config.plugins.PluginConfiguration;
 import org.apache.logging.log4j.core.config.plugins.PluginElement;
 import org.apache.logging.log4j.core.config.plugins.PluginFactory;
+import org.apache.logging.log4j.core.impl.LocationAware;
 import org.apache.logging.log4j.core.pattern.LogEventPatternConverter;
 import org.apache.logging.log4j.core.pattern.PatternFormatter;
 import org.apache.logging.log4j.core.pattern.PatternParser;
@@ -142,6 +143,12 @@ public final class PatternLayout extends AbstractStringLayout {
         return new SerializerBuilder();
     }
 
+    @Override
+    public boolean requiresLocation() {
+        return eventSerializer instanceof LocationAware && ((LocationAware) eventSerializer).requiresLocation();
+    }
+
+
     /**
      * Deprecated, use {@link #newSerializerBuilder()} instead.
      *
@@ -209,6 +216,10 @@ public final class PatternLayout extends AbstractStringLayout {
         return eventSerializer.toSerializable(event);
     }
 
+    public void serialize(final LogEvent event, StringBuilder stringBuilder) {
+        eventSerializer.toSerializable(event, stringBuilder);
+    }
+
     @Override
     public void encode(final LogEvent event, final ByteBufferDestination destination) {
         if (!(eventSerializer instanceof Serializer2)) {
@@ -306,7 +317,7 @@ public final class PatternLayout extends AbstractStringLayout {
             .build();
     }
 
-    private static class PatternSerializer implements Serializer, Serializer2 {
+    private static class PatternSerializer implements Serializer, Serializer2, LocationAware {
 
         private final PatternFormatter[] formatters;
         private final RegexReplacement replace;
@@ -342,6 +353,16 @@ public final class PatternLayout extends AbstractStringLayout {
             return buffer;
         }
 
+        @Override
+        public boolean requiresLocation() {
+            for (PatternFormatter formatter : formatters) {
+                if (formatter.requiresLocation()) {
+                    return true;
+                }
+            }
+            return false;
+        }
+
         @Override
         public String toString() {
             final StringBuilder builder = new StringBuilder();
@@ -427,7 +448,7 @@ public final class PatternLayout extends AbstractStringLayout {
 
     }
 
-    private static class PatternSelectorSerializer implements Serializer, Serializer2 {
+    private static class PatternSelectorSerializer implements Serializer, Serializer2, LocationAware {
 
         private final PatternSelector patternSelector;
         private final RegexReplacement replace;
@@ -464,6 +485,11 @@ public final class PatternLayout extends AbstractStringLayout {
             return buffer;
         }
 
+        @Override
+        public boolean requiresLocation() {
+            return patternSelector instanceof LocationAware && ((LocationAware) patternSelector).requiresLocation();
+        }
+
         @Override
         public String toString() {
             final StringBuilder builder = new StringBuilder();
